package com.enteprise.attract_inv.service.impl;

import com.enteprise.attract_inv.domain.AttractInv;
import com.enteprise.attract_inv.domain.dto.AttractInvDto;
import com.enteprise.attract_inv.domain.dto.AttractInvReturnDto;
import com.enteprise.attract_inv.service.IAttractInvService;
import com.enteprise.base_data_year.domain.EnterpriseBaseDataYear;
import com.enteprise.base_data_year.mapper.EnterpriseBaseDataYearMapper;
import com.enteprise.enterprise.domain.Enterprise;
import com.enteprise.enterprise.dto.EnterpriseDto;
import com.enteprise.enterprise.mapper.EnterpriseMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class AttractInvServiceImpl implements IAttractInvService {
    @Autowired
    private EnterpriseBaseDataYearMapper enterpriseBaseDataYearMapper;
    @Autowired
    private EnterpriseMapper enterpriseMapper;
    @Override
    public AttractInvReturnDto getData(AttractInv attractInv) {
        AttractInvReturnDto attractInvReturnDto = new AttractInvReturnDto();
        String code = attractInv.getCode();
        String typeNum = attractInv.getTypeNum();
        int year = attractInv.getYear();
        //获取行业总产值和行业总用地
        List<Long> aList = new ArrayList<>();
        List<Long> bList = new ArrayList<>();
        double averageAnnualOutputValue = 0;
        double averageAnnualTaxPaid = 0;
        for (int i = 1; i<4; i++) {
            EnterpriseBaseDataYear dataYear = new EnterpriseBaseDataYear();
            //近三年行业数据
            Long areaSum = 0L;
            Long taxSum = 0L;
            Long industrialSum = 0L;
            dataYear.setYear(String.valueOf(year - i));
            dataYear.setCode(code);
            List<EnterpriseBaseDataYear> yearList = enterpriseBaseDataYearMapper.selectEnterpriseBaseDataYearList(dataYear);
            for (EnterpriseBaseDataYear item : yearList){
                areaSum += item.getLandArea();
                taxSum += item.getPaidTax();
                industrialSum += item.getTotalIndustrialValue();
            }
            if (areaSum != 0  ){
                aList.add(taxSum / areaSum);
                bList.add(industrialSum / areaSum);
            }
        }
        if (!aList.isEmpty()){
            for (Long a : aList){
                averageAnnualOutputValue += a;
            }
            //基准值
            attractInvReturnDto.setStandardA(averageAnnualOutputValue / aList.size() * 1.5);
        }
        if (!bList.isEmpty()){
            for (Long b : bList){
                averageAnnualTaxPaid += b;
            }
            attractInvReturnDto.setStandardB(averageAnnualTaxPaid / bList.size() * 1.5);
        }


        //筛选企业
        ArrayList<AttractInvDto> attractInvDtos = new ArrayList<>();
        Enterprise enterprise = new Enterprise();
        enterprise.setTypeNum(typeNum);
        enterprise.setCode(code);
        List<EnterpriseDto> enterpriseDtos = enterpriseMapper.selectAllWithType(enterprise);
        //计算各企业的近三年
        for (EnterpriseDto dto : enterpriseDtos){
            AttractInvDto attractInvDto = new AttractInvDto();
            String enterpriseName = dto.getEnterpriseName();
            attractInvDto.setId(dto.getId());
            attractInvDto.setTypeNum(dto.getTypeNum());
            attractInvDto.setEnterpriseName(enterpriseName);
            //近三年企业数据
            double areaSum = 0;
            double taxSum = 0;
            double industrialSum = 0;
            int divisorA = 0;
            int divisorB = 0;
            for (int i = 1; i<4; i++) {
                EnterpriseBaseDataYear dataYear = new EnterpriseBaseDataYear();
                dataYear.setYear(String.valueOf(year - i));
                dataYear.setEnterpriseName(enterpriseName);
                List<EnterpriseBaseDataYear> yearList = enterpriseBaseDataYearMapper.selectEnterpriseBaseDataYearList(dataYear);
                if(!yearList.isEmpty()) {
                    areaSum += yearList.get(0).getLandArea();
                    Long tax = yearList.get(0).getPaidTax();
                    taxSum += tax;
                    Long industrialValue = yearList.get(0).getTotalIndustrialValue();
                    industrialSum += industrialValue;

                    if (tax != 0) {
                        divisorA += 1;
                    }
                    if (industrialValue != 0) {
                        divisorB += 1;
                    }
                }
            }
            //计算该企业近三年的用地、年均产值、年均实缴税金
            if (divisorB !=0) {
                attractInvDto.setAverageAnnualOutputValue(industrialSum / divisorB);
            }
            if (divisorA != 0) {
                attractInvDto.setAverageAnnualTaxPaid(taxSum / divisorA);
            }
            attractInvDto.setLandUsedInRecentThreeYears(areaSum);
            //添加到返回list
            attractInvDtos.add(attractInvDto);
        }
        attractInvReturnDto.setList(attractInvDtos);
        return attractInvReturnDto;
    }
}
